/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.conqat.ide.commons.ui.extension.ExtensionPointUtils;
import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.conqat.lib.commons.reflect.ReflectionUtils;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.interfaces.ICompositionContext;
import org.fortiss.tooling.kernel.interfaces.ICompositor;
import org.fortiss.tooling.kernel.services.ICompositorService;
import org.osgi.framework.Bundle;

/**
 * This class implements the {@link ICompositorService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public class CompositorService implements ICompositorService {

	/** The compositor extension point ID. */
	private static final String EXTENSION_POINT_NAME = "org.fortiss.tooling.kernel.modelElementCompositor";

	/** The compositor configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "modelElementCompositor";

	/** Stores the compositors for each model element class. */
	private Map<Class<?>, List<ICompositor<EObject>>> compositorMap;

	/** Constructor. */
	public CompositorService() {
		setupCompositorMap();
	}

	/** Initializes the compositor map from plugin extensions. */
	@SuppressWarnings({ "unchecked" })
	private void setupCompositorMap() {
		compositorMap = new HashMap<Class<?>, List<ICompositor<EObject>>>();
		for (IConfigurationElement ce : ExtensionPointUtils
				.getConfigurationElements(EXTENSION_POINT_NAME,
						CONFIGURATION_ELEMENT_NAME)) {
			Bundle bundle = ExtensionPointUtils.getBundle(ce);
			try {
				Class<?> modelElementClass = ExtensionPointUtils.loadClass(
						ce.getAttribute("modelElementClass"), bundle);
				Class<?> compositorClass = ExtensionPointUtils.loadClass(
						ce.getAttribute("compositor"), bundle);
				ICompositor<EObject> compositor = (ICompositor<EObject>) compositorClass
						.getConstructor().newInstance();
				addCompositor(modelElementClass, compositor);
			} catch (Exception ex) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						ex.getMessage(), ex);
			}
		}
	}

	/** Adds the given compositor to the map. */
	private void addCompositor(Class<?> modelElementClass,
			ICompositor<EObject> compositor) {
		List<ICompositor<EObject>> list = compositorMap.get(modelElementClass);
		if (list == null) {
			list = new LinkedList<ICompositor<EObject>>();
			compositorMap.put(modelElementClass, list);
		}
		list.add(compositor);
	}

	/** {@inheritDoc} */
	@Override
	public boolean canCompose(EObject container, EObject element,
			ICompositionContext context) {
		return findWorkingCompositor(container, element, context) != null;
	}

	/** {@inheritDoc} */
	@Override
	public boolean compose(EObject container, EObject element,
			ICompositionContext context) {
		return findWorkingCompositor(container, element, context).compose(
				container, element, context);
	}

	/**
	 * Returns the first {@link ICompositor} which can compose the given element
	 * (or <code>null</code>).
	 */
	private ICompositor<EObject> findWorkingCompositor(EObject container,
			EObject contained, ICompositionContext context) {
		final List<ICompositor<EObject>> list = ReflectionUtils
				.performNearestClassLookup(container.getClass(), compositorMap);
		if (list == null) {
			return null;
		}
		for (ICompositor<EObject> compositor : list) {
			if (compositor.canCompose(container, contained, context)) {
				return compositor;
			}
		}
		return null;
	}
}
